  include 'blocks.f90'
  include 'formats.f90'
  include 'constants.f90'
  program PF_eta
    use blocks
    use formats
    implicit none
      
    real*8    :: p, dp, RL, RL_p, RL_0, fdist, error
    real*8    :: kappa_min, kappa_max, dkappa, eta

    integer*4 :: info, ip, jp, ip_fail, iter

    logical*4 :: flag

    real*8, allocatable, dimension(:) :: z, y, F

    external check_pars, update_z, make_F, fdist

    Job = 'PF_eta'

    write(6,f1) '=========================================='
    write(6,f1) '=========================================='
    write(6,f1) '=========================================='
    call check_pars(info)
    write(6,f1) '******************************************'
    if(info .eq. 0) then
       write(6,f1) 'PF_eta: Parameters checked for sanity'
    else if(info .gt. 0) then
       write(6,f2) 'PF_eta: There were errors on input, info=', info
       stop
    else 
       write(6,f2) 'PF_eta: There were warnings on input, info=', info
    end if

    kappa_min = rz
    kappa_max = kappa
    dkappa    = (kappa_max-kappa_min)/dfloat(Np-1)
    dp        = p_max
    if(dp .lt. small) then
       write(6,f5) 'PF_eta: dp<=small (fatal); dp=', dp
       stop
    end if
      
    write(6,f1) 'PF_eta computes Youngs modulus eta as functions of kappa at e=0'
    write(6,f1) 'File  "eta.dat" : kappa, eta*h/k'
    write(6,f1) '... where'
    write(6,f1) 'eta*h/k = dp / [1 - L(dp)/L(0)] -- Eq.46'
    write(6,f3) '[set by code         ] -- kappa_min: ', sngl(rz)
    write(6,f3) '[from parameter kappa] -- kappa_max: ', sngl(kappa_max)
    write(6,f3) '[from parameter p_max] --        dp: ', sngl(dp)
    write(6,f1) '******************************************'

    allocate(z(1:N))
    allocate(y(1:N))
    allocate(F(1:N))

    open(unit=70, file='eta.dat', status='unknown')

    flag = .false.

    ip_loop: do ip=1,Np
       kappa = kappa_min + dkappa*dfloat(ip-1)

       jp_loop: do jp=1,2
          if     (jp .eq. 1) then
             p = rz
          else if(jp .eq. 2) then
             p = dp
          else
             write(6,f1) 'PF_eta: Unexpected error at Point A (fatal)'
             stop
          end if
         
          F = rz; F(1) = p; F(N) = -p
          call update_z(F, z, error)
          if(error .gt. small) then
             write(6,f5) 'PF_eta: Error in update_z at Point B:', error
             flag = .true.
             ip_fail = ip
             exit ip_loop
          end if

          iter = 0
          do_loop: do
             iter = iter + 1
             if(iter .gt. iter_max) then
                write(6,f1) 'PF_eta: maximum number of iterations has been reached'
                flag = .true.
                ip_fail = ip
                exit ip_loop
             end if
             call make_F(z,F)
             F(1) = F(1) + p; F(N) = F(N) - p
             y    = z
             call update_z(F, z, error)
             if(error .gt. small) then
                write(6,f5) 'PF_eta: Error in update_z at Point C:', error
                flag = .true.
                ip_fail = ip
                exit ip_loop
             end if
             if(fdist(z,y) .le. th1) exit do_loop
          end do do_loop

          RL = z(N) - z(1)

          if     (jp .eq. 1) then
             RL_0 = RL
          else if(jp .eq. 2) then
             RL_p = RL
          else
             write(6,f1) 'PF_eta: Unexpected error at Point D'
             stop
          end if

       end do jp_loop

       eta = dp*RL_0 / (RL_0 - RL_p)

       write(70,*) sngl(kappa), sngl(eta)

       if(Vocal) then
          write(6,*) sngl(kappa), sngl(eta), sngl(RL_p/RL_0)
       else
          write(6,fl,advance='NO') CC, '------ Complete : ', ip , ' of ', Np
          flush(6)
       end if
 
    end do ip_loop

    close(unit=70)

    write(*,*)
      
    if(flag) then
       write(6,f4) 'PF_p: all data points up to ip=', ip_fail, ' were computed successfully'
    else
       write(6,f1) 'PF_p: all data points computed successfully'
    end if

    stop
  end program PF_eta
